//==============================================================================
// Thierry Coq, 2008-02-23
// Small FPC example for creating a test library, and load it dynamically.
// Use it at your risk. Enjoy.
// This unit provides the interface to the OS Loading and unloading
// functions.
// - Windows : DONE.
//   See http://en.wikipedia.org/wiki/Dynamic-link_library for details.
// - Linux : TODO.
//   It shouldn't be too difficult... ;-).
//==============================================================================
unit DynamicLoadingLibrary;

{$mode objfpc}{$H+}

interface

Type
  THandle           = LongWord;
  THandleLibrary    = THandle;
  TPointerChar      = PChar;
  TProcedurePointer = Pointer;

//Function to load a DLL
//loadlibrary is taken from windows, to get the lib
function LoadLibrary(aLibName: PChar): THandleLibrary; stdcall;

//This is the opposite function.
function UnloadLibrary(aLibHandle: THandleLibrary): LongBool; stdcall;

//Function to find the address of a procedure or function, based on its name.
//(Case sensitive...)
function GetProcAddress(aLibHandle: THandleLibrary; aProcName: PChar): TProcedurePointer; stdcall;


implementation

const
  Kernel32DLL  = 'kernel32.dll';


function LoadLibrary(aLibName: PChar): THandleLibrary; stdcall;
  external Kernel32DLL name 'LoadLibraryA';

function UnloadLibrary(aLibHandle: THandleLibrary): LongBool; stdcall;
  external Kernel32DLL name 'FreeLibrary';
  //Note: why didn't Kro name this function Unload... is unfathomable...

function GetProcAddress(aLibHandle: THandleLibrary; aProcName: PChar): TProcedurePointer; stdcall;
  external Kernel32DLL name 'GetProcAddress';

end.

