//==============================================================================
// Thierry Coq, 2008-02-23
// Small FPC example for creating a test library, and load it dynamically.
// Use it at your risk. Enjoy.
//==============================================================================
unit TestInterface;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils;

//This function interfaces the <real> call to the DLL. It is only a placeholder
//to make it more easy to load the function.
Function AddInteger( I1, I2 : Integer) : Integer;

//==============================================================================
implementation

uses
  DynamicLoadingLibrary;

//In order for the function to be used, its type must be defined.
//Beware: it is sort of a pointer type, so it's not needed to add another pointer
//type. It mislead my for awhile.
Type
  TFunctionAddInteger = Function (I1, I2:Integer):Integer; stdcall;
  //NOTNEEDED: PFunctionAddInteger = ^TFunctionAddInteger;
  
//Variables to maintain the loaded state.
Var
  Loaded : Boolean;
  PTestLibrary: THandleLibrary;
  PAddInteger : TFunctionAddInteger;
  
// Test function to check the calling protocol, without actually loading
// the DLL. To use declare, set CheckCallingProtocol to true.
Const CheckCallingProtocol : Boolean = False;
Function TestAddInteger(I1, I2: Integer): Integer; Stdcall;
begin
  result := I1 +I2;
end;

// Loads the DLL, and the test function.
procedure Load;
begin
  if Loaded then Exit;
  PTestLibrary := LoadLibrary( 'TestLibrary.dll');
  if Not CheckCallingProtocol then
    PAddInteger  := TFunctionAddInteger( GetProcAddress(PTestLibrary, 'AddInteger'))
  else
    PAddInteger  := @TestAddInteger;
  Loaded := True;
end;

// Unloading the DLL
procedure Unload;
begin
  if Loaded then
  begin
    //TODO : unload DLL...
    PAddInteger := Nil;
    UnloadLibrary( PTestLibrary);
    Loaded := False;
  end;
end;

Function AddInteger( I1, I2 : Integer) : Integer;
begin
  Load;
  result := PAddInteger(I1, I2);
end;

//==============================================================================
initialization
  Loaded := False;
//==============================================================================
finalization
  UnLoad;
end.

